import 'package:flutter/foundation.dart';

class LVSKUCondition {
  List<LVSKUProperty> _properties = [];

  List<int> conditionIndexs = [];
  Set<LVIndexPath> conditionIndexPaths = {};

  dynamic result;

  set setProperties(List<LVSKUProperty> properties) {
    _properties = properties;

    conditionIndexs.clear();
    conditionIndexPaths.clear();
    properties.forEach((element) {
      conditionIndexs.add(element.indexPath.row);
      conditionIndexPaths.add(element.indexPath);
    });
  }
}

class LVSKUProperty {
  LVIndexPath indexPath;
  dynamic value;

  LVSKUProperty({required this.indexPath, required this.value});
}

class LVIndexPath {
  LVIndexPath({
    this.section = 0,
    this.row = 0,
  });

  int section;
  int row;

  @override
  bool operator ==(Object other) {
    if (other is! LVIndexPath) {
      return false;
    }
    final LVIndexPath indexPath = other;
    return section == indexPath.section && row == indexPath.row;
  }

  @override
  int get hashCode {
    var tempHashCode = 17;
    tempHashCode = 37 * tempHashCode + section.hashCode;
    tempHashCode = 37 * tempHashCode + row.hashCode;
    return tempHashCode;
  }
}

class LVSKUDataFilter {
  LVSKUDataFilter({
    this.needDefaultValue = false,
    required int numberOfSections,
    required int numberOfConditions,
    required List Function(int row) conditionForRow,
    required dynamic Function(int row) resultOfConditionForRow,
    required List Function(int section) propertiesInSection,
  }) {
    _numberOfSections = numberOfSections;
    _propertiesInSection = propertiesInSection;

    Set<LVSKUCondition> modelSet = {};

    for (int i = 0; i < numberOfConditions; i++) {
      LVSKUCondition model = LVSKUCondition();
      List conditions = conditionForRow(i);

      if (!_checkConformToSkuConditions(conditions)) {
        debugPrint("第 $i 个 condition 不完整 \n $conditions");
        continue;
      }

      model.setProperties = _propertiesWithConditionRawData(conditions);
      model.result = resultOfConditionForRow(i);
      print(
          """555555   resultOfConditionForRow(i) = ${resultOfConditionForRow(i)}
          model.result = ${model.result}
          """);

      if (selectedIndexPaths.isEmpty &&
          needDefaultValue &&
          _defaultSku == null) {
        _defaultSku = model;
      }

      modelSet.add(model);
    }

    _conditions = Set.from(modelSet);

    _getAllAvailableIndexPaths();

    if (_defaultSku != null) {
      _defaultSku!._properties.forEach((element) {
        didSelectedPropertyWithIndexPath(element.indexPath);
      });
    }
  }

  late int _numberOfSections;
  late List Function(int section) _propertiesInSection;

  bool needDefaultValue;
  LVSKUCondition? _defaultSku;

  late Set<LVSKUCondition> _conditions;
  Set<LVIndexPath> selectedIndexPaths = <LVIndexPath>{};
  late Set<LVIndexPath> availableIndexPathsSet;
  late Set<LVIndexPath> _allAvailableIndexPaths;
  var currentResult;

  //选中某个属性
  didSelectedPropertyWithIndexPath(LVIndexPath indexPath) {
    if (!availableIndexPathsSet.contains(indexPath)) {
      //不可选
      return;
    }
    if (indexPath.section > _numberOfSections ||
        indexPath.row >= _propertiesInSection(indexPath.section).length) {
      //越界
      debugPrint("indexPath is out of range");
      return;
    }

    if (selectedIndexPaths.contains(indexPath)) {
      //已被选
      selectedIndexPaths.remove(indexPath);

      updateAvailableIndexPaths();
      print("updateCurrentResult");
      updateCurrentResult();
      return;
    }

    LVIndexPath? lastIndexPath;

    selectedIndexPaths.forEach((element) {
      if (indexPath.section == element.section) {
        lastIndexPath = element;
      }
    });

    if (lastIndexPath == null) {
      //添加新属性
      selectedIndexPaths.add(indexPath);
      availableIndexPathsSet = availableIndexPathsSet
          .intersection(availableIndexPaths(indexPath, selectedIndexPaths));
      updateCurrentResult();
      return;
    }

    if (lastIndexPath!.row != indexPath.row) {
      //切换属性
      selectedIndexPaths.add(indexPath);
      selectedIndexPaths.remove(lastIndexPath);
      updateAvailableIndexPaths();
      updateCurrentResult();
    }
  }
}

extension LVSKUDataFilterExtensionForPrivateMethods on LVSKUDataFilter {
  //检查数据是否正确
  bool _checkConformToSkuConditions(List conditions) {
    // todo: 待完善
    return true;
  }

  //获取属性
  List<LVSKUProperty> _propertiesWithConditionRawData(List data) {
    return data.asMap().keys.map((index) {
      return _property(data[index], index);
    }).toList();
  }

  LVSKUProperty _property(dynamic value, int section) {
    List properties = _propertiesInSection(section);

    assert(properties.contains(value),
        "Properties for $section dosen‘t exist $value");

    LVIndexPath indexPath =
        LVIndexPath(section: section, row: properties.indexOf(value));

    return LVSKUProperty(value: value, indexPath: indexPath);
  }

//获取初始可选的所有IndexPath
  Set<LVIndexPath> _getAllAvailableIndexPaths() {
    Set<LVIndexPath> set = {};

    _conditions.forEach((element) {
      element.conditionIndexs.asMap().forEach((index, value) {
        set.add(LVIndexPath(row: value.toInt(), section: index));
      });
    });

    availableIndexPathsSet = set;

    _allAvailableIndexPaths = Set.from(set);

    return set;
  }

  //当前可用的
  updateAvailableIndexPaths() {
    if (selectedIndexPaths.length == 0) {
      availableIndexPathsSet = Set.from(_allAvailableIndexPaths);
      return;
    }

    Set<LVIndexPath> set = {};

    Set<LVIndexPath> seleted = {};

    selectedIndexPaths.forEach((element) {
      seleted.add(element);

      Set tempSet = availableIndexPaths(element, seleted);

      if (set.length == 0) {
        set = Set.from(tempSet);
      } else {
        set = set.intersection(tempSet);
      }
    });

    availableIndexPathsSet = set;
    print("444444updateAvailableIndexPaths()");
  }

// 当前结果
  updateCurrentResult() {
    if (selectedIndexPaths.length != _numberOfSections) {
      currentResult = null;
      return;
    }

    currentResult = _conditions.firstWhere((element) {
      return element.conditionIndexPaths.containsAll(selectedIndexPaths) &&
          selectedIndexPaths.containsAll(element.conditionIndexPaths);
    }).result;

    print("""
    44444444  updateCurrentResult
    _conditions = $_conditions
    currentCondition = ${_conditions.firstWhere((element) {
      return element.conditionIndexPaths.containsAll(selectedIndexPaths) &&
          selectedIndexPaths.containsAll(element.conditionIndexPaths);
    })}
    currentResult = ${_conditions.firstWhere((element) {
      return element.conditionIndexPaths.containsAll(selectedIndexPaths) &&
          selectedIndexPaths.containsAll(element.conditionIndexPaths);
    }).result}
    """);
  }

  //选中某个属性时 根据已选中的系列属性 获取可选的IndexPath
  Set<LVIndexPath> availableIndexPaths(
      LVIndexPath selectedIndexPath, Set<LVIndexPath> sectedIndexPaths) {
    Set<LVIndexPath> set = {};

    _conditions.forEach((obj) {
      if (obj.conditionIndexs[selectedIndexPath.section] ==
          selectedIndexPath.row) {
        obj._properties.forEach((property) {
          //从condition中添加种类不同的属性时，需要根据已选中的属性过滤
          //过滤方式为 condition要么包含已选中 要么和已选中属性是同级
          if (property.indexPath.section != selectedIndexPath.section) {
            bool flag = true;

            selectedIndexPaths.forEach((obj1) {
              flag = ((obj.conditionIndexs[obj1.section] == obj1.row) ||
                      (obj1.section == property.indexPath.section)) &&
                  flag;
            });

            if (flag) {
              set.add(property.indexPath);
            }
          } else {
            set.add(property.indexPath);
          }
        });
      }
    });

    //合并本行数据
    _allAvailableIndexPaths.forEach((obj) {
      if (obj.section == selectedIndexPath.section) {
        set.add(obj);
      }
    });

    return set;
  }
}
